@@ -12,17 +12,17 @@
 /* .nf
 
  /*
-  * System library.
+  * Utility library.
   */
-#include <netinet/in.h>
+#include <inet_addr_list.h>
 
  /*
   * External interface.
   */
-extern int own_inet_addr(struct in_addr *);
+extern int own_inet_addr(struct sockaddr *);
 extern struct INET_ADDR_LIST *own_inet_addr_list(void);
 extern struct INET_ADDR_LIST *own_inet_mask_list(void);
-extern int proxy_inet_addr(struct in_addr *);
+extern int proxy_inet_addr(struct sockaddr *);
 extern struct INET_ADDR_LIST *proxy_inet_addr_list(void);
 
 /* LICENSE
