@@ -23,7 +23,7 @@
 #define TLS_MGR_REQ_LOOKUP	"lookup"
 #define TLS_MGR_REQ_UPDATE	"update"
 #define TLS_MGR_REQ_DELETE	"delete"
-#define TLS_MGR_ATTR_POLICY	"policy"
+#define TLS_MGR_ATTR_CACHABLE	"cachable"
 #define TLS_MGR_ATTR_CACHE_TYPE	"cache_type"
 #define TLS_MGR_ATTR_SEED	"seed"
 #define TLS_MGR_ATTR_CACHE_ID	"cache_id"
@@ -38,20 +38,14 @@
 #define TLS_MGR_STAT_ERR	(-1)	/* object not found */
 #define TLS_MGR_STAT_FAIL	(-2)	/* protocol error */
 
- /*
-  * Are we talking about the client or server cache?
-  */
-#define TLS_MGR_SCACHE_CLIENT	(1<<0)
-#define TLS_MGR_SCACHE_SERVER	(1<<1)
-
  /*
   * Functional interface.
   */
 extern int tls_mgr_seed(VSTRING *, int);
-extern int tls_mgr_policy(int *);
-extern int tls_mgr_lookup(int, const char *, VSTRING *);
-extern int tls_mgr_update(int, const char *, const char *, int);
-extern int tls_mgr_delete(int, const char *);
+extern int tls_mgr_policy(const char *, int *);
+extern int tls_mgr_lookup(const char *, const char *, VSTRING *);
+extern int tls_mgr_update(const char *, const char *, const char *, ssize_t);
+extern int tls_mgr_delete(const char *, const char *);
 
 /* LICENSE
 /* .ad
